/**
 * @brief Master Telescope Driver
 * @author 
 * @date May 10, 2020
 * @version 1.0
 * 
*/

#include "simpletools.h"
#include "fdserial.h"

void readModePin(void);
void readDirectionPins(void);
void sendSerialData(void);

#define westPin    0
#define northPin   1
#define eastPin    2
#define southPin   3
#define modePin    4
#define lowLedPin  7
#define med1LedPin 8
#define med2LedPin 9
#define highLedPin 10
#define rmotorPin  11
#define rdirPin    12
#define dmotorPin  13
#define ddirPin    14

#define rx 5
#define tx 6

char directionFlag = 0;
char westPinState = 0;
char lastwestPinState = 0;
char northPinState = 0;
char lastnorthPinState = 0;
char eastPinState = 0;
char lasteastPinState = 0;
char southPinState = 0;
char lastsouthPinState = 0;
char currentModeSwitchState = 0;
char lastModeSwitchState = 0;
char mode;
unsigned long currentMillis;
unsigned long lastMillis;
volatile char RmotorPin = 0;
volatile char DmotorPin = 0;
volatile char RdirPin = 0;
volatile char DdirPin = 0;

fdserial *fd;


int main()
{
  
  low(lowLedPin);
  low(med1LedPin);
  low(med2LedPin);
  low(highLedPin);
  
  low(rmotorPin);
  low(rdirPin);
  low(dmotorPin);
  low(ddirPin);
  
  fd = fdserial_open(rx, tx, FDSERIAL_MODE_NONE, 115200);
  
  while(1)
  {
    readModePin();
    readDirectionPins();
    pause(50); // wait 50 millis
  }
}
 //                         readModePin 
 //***************************************************************//
 void readModePin()
  {
    currentModeSwitchState = input(modePin);
    if(currentModeSwitchState != lastModeSwitchState)
    {
      lastModeSwitchState = currentModeSwitchState;
      if(currentModeSwitchState == 0)
       {
        mode ++;
        if(mode > 4)
         {
          mode = 1;
         }
       }
      else
      {}
    }
     switch(mode)
      {
        case 1:
          high(lowLedPin);
          low(med1LedPin);
          low(med2LedPin);
          low(highLedPin);
          break;

        case  2:
          low(lowLedPin);
          high(med1LedPin);
          low(med2LedPin);
          low(highLedPin);
          break;

        case  3:
          low(lowLedPin);
          low(med1LedPin);
          high(med2LedPin);
          low(highLedPin);
          break;
        
        case  4:
          low(lowLedPin);
          low(med1LedPin);
          low(med2LedPin);
          high(highLedPin);
          break;

        default:
          {}
      }
  }

 //**                                     readDirectionalPins
 //******************************************************************//
 void readDirectionPins()
   {     //0 = none,  1 = west,  2 = north,  3 = east,  4 = south,  5 = reset
     westPinState = input(westPin);
     if(westPinState != lastwestPinState)
      {
        lastwestPinState = westPinState;
        if(westPinState == 0)
         {
          directionFlag = 1;
          RdirPin = 0;
          DdirPin = 0;
         }
        else
         {
          directionFlag = 5;
          RmotorPin = 0;
          DmotorPin = 0;
          RdirPin = 0;
          DdirPin = 0;
         }
      }
 //********************************************************************************//
     northPinState = input(northPin);
     if(northPinState != lastnorthPinState)
      {
        lastnorthPinState = northPinState;
        if(northPinState == 0)
         {
           directionFlag = 2;
           RdirPin = 0;
           DdirPin = 0;
         }
        else
         {
           directionFlag = 5;
           RmotorPin = 0;
           DmotorPin = 0;
           RdirPin = 0;
           DdirPin = 0;
         }
      }
  //*************************************************************************************//
     eastPinState = input(eastPin);
     if(eastPinState != lasteastPinState)
      {
        lasteastPinState = eastPinState;
        if(eastPinState ==0)
         {
           directionFlag = 3;
           RdirPin = 1;
           DdirPin = 0;
         }
        else
         {
           directionFlag = 5;
           RmotorPin = 0;
           DmotorPin = 0;
           RdirPin =0;
           DdirPin = 0;
         }
      }
 //**********************************************************************//
     southPinState = input(southPin);
     if(southPinState != lastsouthPinState)
      {
        lastsouthPinState = southPinState;
        if(southPinState == 0)
         {
          directionFlag = 4;
          RdirPin = 0;
          DdirPin = 1;
         }
        else
         {
           directionFlag = 5;
           RmotorPin = 0;
           DmotorPin = 0;
           RdirPin = 0;
           DdirPin = 0;
         }
      }
 //******************************************************************************//
   if(directionFlag != 0)
    {
      switch (mode)
       {
        // mode 1
        case 1:
         switch (directionFlag)
          {
            // mode 1
            case 1:
             {
               RmotorPin     = 20;
               DmotorPin     =  0;
               directionFlag = 0;
             }
               break;
            case 2:
             {
               RmotorPin     =   0;
               DmotorPin     =  20;
               directionFlag =   0; 
             }
               break;
            case 3:
             {
               RmotorPin     = 20;
               DmotorPin     =  0;
               directionFlag =  0;
             }
               break;
            case 4:
             {
               RmotorPin     =   0;
               DmotorPin     =  20;
               directionFlag =   0;
             }
               break;
          }
            break;
 //***************************************************************************//
    // mode 2:
    case 2:
     switch(directionFlag)
      {  // mode 2 west
         case 1:
         {
           RmotorPin     = 40;
           DmotorPin     =  0;
           directionFlag =  0;
         }
           break;
         case 2:
         {
           RmotorPin     =  0;
           DmotorPin     = 40;
           directionFlag =  0;
         }
           break;
         case 3:
         {
           RmotorPin     = 40;
           DmotorPin     =  0;
           directionFlag =  0;
         }
           break;
         case 4:
         {
           RmotorPin     =  0;
           DmotorPin     = 40;
           directionFlag =  0;
         }
           break;
       }
         break;
 //********************************************************************************//
    //  mode 3:
    case 3:
     switch(directionFlag)
      {  // mode 3 west
         case 1:
         {
           RmotorPin     = 60;
           DmotorPin     =  0;
           directionFlag =  0;
         }
           break;
         case 2:
         {
           RmotorPin     =  0;
           DmotorPin     = 60;
           directionFlag =  0;
         }
           break;
         case 3:
         {
           RmotorPin     = 60;
           DmotorPin     =  0;
           directionFlag =  0;
         }
           break;
         case 4:
         {
           RmotorPin     =  0;
           DmotorPin     = 60;
           directionFlag =  0;
         }
           break;
      }
        break;
 //*********************************************************************************//
    //  mode 4:
    case 4:
     switch(directionFlag)
     {  //  west
        case 1:
        {
          RmotorPin     = 80;
          DmotorPin     =  0;
          directionFlag =  0;   
        }
          break;
        case 2:
        {
          RmotorPin     =  0;
          DmotorPin     = 80;
          directionFlag =  0;
        }
           break;
        case 3:
        {
          RmotorPin     = 80;
          DmotorPin     =  0;
          directionFlag =  0;
        }
          break;
        
         case 4:
         {
           RmotorPin     =  0;
           DmotorPin     = 80;
           directionFlag =  0;
         }
           break;
       }
         break;   
     }
      
//************************************************************************************//
    if(directionFlag ==5)
     {
      directionFlag = 0;
     }
     sendSerialData();
   }
 }
 
//************************************************************************************//
void sendSerialData()
{

  fdserial_txChar(fd, RmotorPin);
  fdserial_txChar(fd, RdirPin);
  fdserial_txChar(fd, DmotorPin);
  fdserial_txChar(fd, DdirPin);

  printi("value1: %d, value2: %d value3: %d, value4: %d\n", RmotorPin, RdirPin, DmotorPin, DdirPin);
     
}